use std::{os::unix::prelude::PermissionsExt, path::Path};

use anyhow::{anyhow, Context};
use reqwest::Url;

const GET_PRYMN_URL: &str = "todo";

pub fn update() -> anyhow::Result<()> {
    let _url = {
        let url = std::env::var("GET_PRYMN_URL").unwrap_or_else(|_| String::from(GET_PRYMN_URL));
        Url::parse(&url)?
    };

    todo!();
}

pub fn install(token: &str) -> anyhow::Result<()> {
    let this_exe = std::env::current_exe()?;
    let prymn_path = Path::new("/usr/local/bin/prymn_agent");

    copy_binary(&this_exe, prymn_path)?;
    register_agent(token).context("while registering the agent")?;
    install_service()?;

    Ok(())
}

fn copy_binary(src: impl AsRef<Path>, dest: impl AsRef<Path>) -> anyhow::Result<()> {
    let src = src.as_ref();
    let dest = dest.as_ref();

    if dest.exists() {
        // unlink the potentially running binary
        std::fs::remove_file(dest)?;
    }

    std::fs::copy(src, dest)?;

    let metadata = dest.metadata()?;
    let mut perms = metadata.permissions();
    perms.set_mode(0o755);
    std::fs::set_permissions(dest, perms)?;

    Ok(())
}

fn register_agent(token: &str) -> anyhow::Result<()> {
    let client = reqwest::blocking::Client::new();

    let res = client
        .post("http://localhost:4000/api/v1/servers/register")
        .body(serde_json::json!({"token": token}).to_string())
        .send()?;

    if !res.status().is_success() {
        // TODO: Make better error message
        return Err(anyhow!(
            "register request returned an error: {}",
            res.text()?
        ));
    }

    Ok(())
}

fn install_service() -> anyhow::Result<()> {
    Ok(())
}
